@@ -9,8 +9,6 @@ import dataclasses
 import enum
 from typing import List, Optional
 
-from simple_parsing import choice
-
 from pynguin.utils.statistics.runtimevariable import RuntimeVariable
 
 
@@ -60,7 +58,55 @@ class Algorithm(str, enum.Enum):
     WHOLE_SUITE = "WHOLE_SUITE"
     """A whole-suite test generation approach similar to the one proposed by EvoSuite
     (cf. Fraser and Arcuri. EvoSuite: Automatic Test Suite Generation for
-    Object-Oriented Software. Proc. ESEC/FSE 2011)."""
+    Object-Oriented Software. Proc. ESEC/FSE 2011).
+
+    This algorithm can be modified to use an archive (cf. Rojas, José Miguel, et al.
+    "A detailed investigation of the effectiveness of whole test suite generation."
+    Empirical Software Engineering 22.2 (2017): 852-893.), by using the
+    following options: --use-archive True, --seed_from_archive True and
+    --filter_covered_targets_from_test_cluster True.
+    """
+
+
+class AssertionGenerator(str, enum.Enum):
+    """Different approaches for assertion generation supported by Pynguin."""
+
+    MUTATION_ANALYSIS = "MUTATION_ANALYSIS"
+    """Use the mutation analysis approach for assertion generation."""
+
+    SIMPLE = "SIMPLE"
+    """Use the simple approach for primitive and none assertion generation."""
+
+    NONE = "NONE"
+    """Do not create any assertions."""
+
+
+class MutationStrategy(str, enum.Enum):
+    """Different strategies for creating mutants when using the MUTATION_ANALYSIS
+    approach for assertion generation."""
+
+    FIRST_ORDER_MUTANTS = "FIRST_ORDER_MUTANTS"
+    """Generate first order mutants."""
+
+    FIRST_TO_LAST = "FIRST_TO_LAST"
+    """Higher order mutation strategy FirstToLast.
+    (cf. Mateo et al. Validating Second-Order Mutation at System Level. Article.
+    IEEE Transactions on SE 39.4 2013)"""
+
+    BETWEEN_OPERATORS = "BETWEEN_OPERATORS"
+    """Higher order mutation strategy BetweenOperators.
+    (cf. Mateo et al. Validating Second-Order Mutation at System Level. Article.
+    IEEE Transactions on SE 39.4 2013)"""
+
+    RANDOM = "RANDOM"
+    """Higher order mutation strategy Random.
+    (cf. Mateo et al. Validating Second-Order Mutation at System Level. Article.
+    IEEE Transactions on SE 39.4 2013)"""
+
+    EACH_CHOICE = "EACH_CHOICE"
+    """Higher order mutation strategy EachChoice.
+    (cf. Mateo et al. Validating Second-Order Mutation at System Level. Article.
+    IEEE Transactions on SE 39.4 2013)"""
 
 
 class StoppingCondition(str, enum.Enum):
@@ -72,8 +118,11 @@ class StoppingCondition(str, enum.Enum):
     MAX_ITERATIONS = "MAX_ITERATIONS"
     """Stop after a maximum number of algorithm iterations."""
 
-    MAX_TESTS = "MAX_TESTS"
-    """Stop as soon as a maximum number of tests was generated."""
+    MAX_TEST_EXECUTIONS = "MAX_TEST_EXECUTIONS"
+    """Stop as soon as a maximum number of tests were executed."""
+
+    MAX_STATEMENT_EXECUTIONS = "MAX_STATEMENT_EXECUTIONS"
+    """Stop as soon as a maximum number of statements were executed."""
 
 
 class TypeInferenceStrategy(str, enum.Enum):
@@ -112,6 +161,7 @@ class Selection(str, enum.Enum):
     """Tournament selection.  Use `tournament_size` to set size."""
 
 
+# pylint:disable=too-many-instance-attributes
 @dataclasses.dataclass
 class StatisticsOutputConfiguration:
     """Configuration related to output."""
@@ -147,6 +197,12 @@ class StatisticsOutputConfiguration:
     """Label that identifies the project name of Pynguin.  This is useful when
     running experiments."""
 
+    create_coverage_report: bool = False
+    """Create a coverage report for the tested module.
+    This can be helpful to find hard to cover parts because Pynguin measures coverage
+    on bytecode level which might yield different results when compared with other
+    tools, e.g., Coverage.py."""
+
 
 @dataclasses.dataclass
 class TestCaseOutputConfiguration:
@@ -163,8 +219,21 @@ class TestCaseOutputConfiguration:
     """The maximum number of statement in as test case (normal + assertion
     statements)"""
 
-    generate_assertions: bool = True
-    """Should assertions be generated?"""
+    assertion_generation: AssertionGenerator = AssertionGenerator.MUTATION_ANALYSIS
+    """The generator that shall be used for assertion generation."""
+
+    mutation_strategy: MutationStrategy = MutationStrategy.FIRST_ORDER_MUTANTS
+    """The strategy that shall be used for creating mutants in the mutation analysis
+    assertion generation method."""
+
+    mutation_order: int = 1
+    """The order of the generated higher order mutants in the mutation analysis
+    assertion generation method."""
+
+    generate_all_assertions: bool = True
+    """Should the mutation analysis approach to assertion generation also generate
+    assertions in cases where the states between non-mutated and mutated modules do
+    not differ?"""
 
     post_process: bool = True
     """Should the results be post processed? For example, truncate test cases after
@@ -213,6 +282,16 @@ class SeedingConfiguration:
     """Probability of using dynamically seeded values when a primitive seeded
      value will be used."""
 
+    seed_from_archive: bool = False
+    """When sampling new test cases reuse some from the archive, if one is used."""
+
+    seed_from_archive_probability: float = 0.2
+    """Instead of creating a new test case, reuse a covering solution from the archive,
+    iff an archive is used."""
+
+    seed_from_archive_mutations: int = 3
+    """Number of mutations applied when sampling from the archive."""
+
 
 @dataclasses.dataclass
 class MIOPhaseConfiguration:
@@ -394,6 +473,15 @@ class SearchAlgorithmConfiguration:
     selection: Selection = Selection.TOURNAMENT_SELECTION
     """The selection operator for genetic algorithms."""
 
+    use_archive: bool = False
+    """Some algorithms can be enhanced with an optional archive, e.g. Whole Suite ->
+    Whole Suite + Archive. Use this option to enable the usage of an archive.
+    Algorithms that always use an archive are not affected by this option."""
+
+    filter_covered_targets_from_test_cluster: bool = False
+    """Focus search by filtering out elements from the test cluster when
+     they are fully covered."""
+
 
 @dataclasses.dataclass
 class StoppingConfiguration:
@@ -405,16 +493,15 @@ class StoppingConfiguration:
     budget: int = 600
     """Time budget (in seconds) that can be used for generating tests."""
 
-    maximum_test_number: int = 60
-    """Maximum number of test cases to be created."""
+    maximum_test_executions: int = 60
+    """Maximum number of test cases to be executed."""
 
-    algorithm_iterations: int = 60
+    maximum_statement_executions: int = 600
+    """Maximum number of test cases to be executed."""
+
+    maximum_iterations: int = 60
     """Maximum iterations"""
 
-    global_timeout: int = 120
-    """Maximum seconds allowed for entire search when not using time as stopping
-    criterion."""
-
 
 # pylint: disable=too-many-instance-attributes, pointless-string-statement
 @dataclasses.dataclass
@@ -430,8 +517,8 @@ class Configuration:
     test_case_output: TestCaseOutputConfiguration
     """Configuration for how test cases should be output."""
 
-    algorithm: Algorithm = choice(Algorithm)
-    """The algorithm that shall be used for generation"""
+    algorithm: Algorithm = Algorithm.DYNAMOSA
+    """The algorithm that shall be used for generation."""
 
     statistics_output: StatisticsOutputConfiguration = dataclasses.field(
         default_factory=StatisticsOutputConfiguration
@@ -472,7 +559,6 @@ class Configuration:
 
 # Singleton instance of the configuration.
 configuration = Configuration(
-    algorithm=Algorithm.RANDOM,
     project_path="",
     module_name="",
     test_case_output=TestCaseOutputConfiguration(output_path=""),
