@@ -10,19 +10,16 @@ from abc import ABCMeta
 from typing import List, cast
 
 import pynguin.configuration as config
-import pynguin.ga.fitnessfunction as ff
 import pynguin.ga.testcasechromosome as tcc
-import pynguin.utils.statistics.statistics as stat
 from pynguin.ga.comparators.dominancecomparator import DominanceComparator
-from pynguin.generation.algorithms.archive import Archive
+from pynguin.generation.algorithms.archive import CoverageArchive
 from pynguin.generation.algorithms.testgenerationstrategy import TestGenerationStrategy
-from pynguin.generation.algorithms.wraptestsuitemixin import WrapTestSuiteMixin
 from pynguin.utils import randomness
 from pynguin.utils.exceptions import ConstructionFailedException
 
 
 class AbstractMOSATestStrategy(
-    TestGenerationStrategy, WrapTestSuiteMixin, metaclass=ABCMeta
+    TestGenerationStrategy[CoverageArchive], metaclass=ABCMeta
 ):
     """An abstract base implementation for MOSA and its derivatives."""
 
@@ -30,9 +27,7 @@ class AbstractMOSATestStrategy(
 
     def __init__(self) -> None:
         super().__init__()
-        self._archive: Archive[ff.FitnessFunction, tcc.TestCaseChromosome]
         self._population: List[tcc.TestCaseChromosome] = []
-        self._current_iteration = 0
         self._number_of_goals = -1
 
     def _breed_next_generation(self) -> List[tcc.TestCaseChromosome]:
@@ -73,10 +68,8 @@ class AbstractMOSATestStrategy(
         ):
             if len(self._archive.covered_goals) == 0 or randomness.next_bool():
                 tch: tcc.TestCaseChromosome = self._chromosome_factory.get_chromosome()
-                for fitness_function in self._fitness_functions:
-                    tch.add_fitness_function(fitness_function)
             else:
-                tch = randomness.choice(list(self._archive.solutions)).clone()
+                tch = randomness.choice(self._archive.solutions).clone()
                 tch.mutate()
 
             if tch.has_changed() and tch.size() > 0:
@@ -120,20 +113,8 @@ class AbstractMOSATestStrategy(
         population: List[tcc.TestCaseChromosome] = []
         for _ in range(config.configuration.search_algorithm.population):
             chromosome = self._chromosome_factory.get_chromosome()
-            for fitness_function in self._fitness_functions:
-                chromosome.add_fitness_function(fitness_function)
             population.append(chromosome)
         return population
 
     def _get_best_individuals(self) -> List[tcc.TestCaseChromosome]:
         return self._get_non_dominated_solutions(self._population)
-
-    def _notify_iteration(self) -> None:
-        test_suite = self.create_test_suite(self._archive.solutions)
-        stat.current_individual(test_suite)
-        coverage = test_suite.get_coverage()
-        self._logger.info(
-            "Generation: %5i. Coverage: %5f",
-            self._current_iteration,
-            coverage,
-        )
