@@ -5,20 +5,25 @@
 #  SPDX-License-Identifier: LGPL-3.0-or-later
 #
 """Provides the MOSA test-generation strategy."""
-import logging
-from typing import List, Set
+from __future__ import annotations
 
-import pynguin.ga.chromosome as chrom
-import pynguin.ga.fitnessfunction as ff
-import pynguin.ga.testcasechromosome as tcc
+import logging
+from typing import TYPE_CHECKING, List
+
+from ordered_set import OrderedSet
+
+import pynguin.ga.computations as ff
 import pynguin.utils.statistics.statistics as stat
 from pynguin.ga.operators.ranking.crowdingdistance import (
     fast_epsilon_dominance_assignment,
 )
 from pynguin.generation.algorithms.abstractmosastrategy import AbstractMOSATestStrategy
-from pynguin.generation.algorithms.archive import Archive
 from pynguin.utils.statistics.runtimevariable import RuntimeVariable
 
+if TYPE_CHECKING:
+    import pynguin.ga.testcasechromosome as tcc
+    import pynguin.ga.testsuitechromosome as tsc
+
 
 # pylint: disable=too-many-instance-attributes
 class MOSATestStrategy(AbstractMOSATestStrategy):
@@ -26,15 +31,13 @@ class MOSATestStrategy(AbstractMOSATestStrategy):
 
     _logger = logging.getLogger(__name__)
 
-    def generate_tests(self) -> chrom.Chromosome:
-        self._logger.info("Start generating tests")
-        self._archive = Archive(set(self._fitness_functions))
-        self._number_of_goals = len(self._fitness_functions)
+    def generate_tests(self) -> tsc.TestSuiteChromosome:
+        self.before_search_start()
+        self._number_of_goals = len(self._test_case_fitness_functions)
         stat.set_output_variable_for_runtime_variable(
             RuntimeVariable.Goals, self._number_of_goals
         )
 
-        self._current_iteration = 0
         self._population = self._get_random_population()
         self._archive.update(self._population)
 
@@ -47,17 +50,17 @@ class MOSATestStrategy(AbstractMOSATestStrategy):
                 fronts.get_sub_front(i), self._archive.uncovered_goals
             )
 
+        self.before_first_search_iteration(
+            self.create_test_suite(self._archive.solutions)
+        )
         while (
-            not self._stopping_condition.is_fulfilled()
+            self.resources_left()
             and self._number_of_goals - len(self._archive.covered_goals) != 0
         ):
             self.evolve()
-            self._notify_iteration()
-            self._current_iteration += 1
+            self.after_search_iteration(self.create_test_suite(self._archive.solutions))
 
-        stat.track_output_variable(
-            RuntimeVariable.AlgorithmIterations, self._current_iteration
-        )
+        self.after_search_finish()
         return self.create_test_suite(
             self._archive.solutions
             if len(self._archive.solutions) > 0
@@ -75,7 +78,7 @@ class MOSATestStrategy(AbstractMOSATestStrategy):
         union.extend(self._population)
         union.extend(offspring_population)
 
-        uncovered_goals: Set[ff.FitnessFunction] = self._archive.uncovered_goals
+        uncovered_goals: OrderedSet[ff.FitnessFunction] = self._archive.uncovered_goals
 
         # Ranking the union
         self._logger.debug("Union Size = %d", len(union))
